// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _FileExists implements Matcher<File> {
  bool matches(File file) => file.existsSync();
  String get description => "be an existing file";
}

class _FileContent implements Matcher<File> {
  Matcher<String> _matcher;

  _FileContent(Matcher<String> this._matcher);

  bool matches(File file) {
    if (!file.existsSync()) {
      return false;
    }

    String content = file.readAsTextSync();
    return _matcher.matches(content);
  }

  String get description => "content ${_matcher.description}";
}

/// Every existing file matches.
Matcher<File> fileExists() => new _FileExists();

/// Every existing file whose content (read as UTF-8 encoded text)
/// matches given [matcher] matches.
Matcher<File> fileContent(Matcher<String> matcher) => new _FileContent(matcher);

