// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

/// Helper base class for implementing [Matcher]s that need to store one value
/// (for comparison etc.).
abstract class SimpleMatcher<T> implements Matcher<T> {
  T savedValue;

  SimpleMatcher(T this.savedValue);
}

// ---

class _IsNull implements Matcher {
  bool matches(value) => value === null;
  String get description => "be null";
}

class _IsNotNull implements Matcher {
  bool matches(value) => value !== null;
  String get description => "not be null";
}

class _Equals<T> extends SimpleMatcher<T> {
  _Equals(T value) : super(value);

  bool matches(T value) => value == savedValue;
  String get description => "be equal to '$savedValue'";
}

/// Only [null] value matches.
Matcher isNull() => new _IsNull();

/// Every object that is not [null] matches.
Matcher isNotNull() => new _IsNotNull();

/// Every object that is equal to given [value] according to the `==` operator matches.
Matcher equals(value) => new _Equals(value);

