// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _And<T> implements Matcher<T> {
  List<Matcher<T>> _delegates;

  _And(List<Matcher<T>> this._delegates);

  bool matches(T value) {
    for (final m in _delegates) {
      if (!m.matches(value)) {
        return false;
      }
    }
    return true;
  }

  String get description => join(_delegates, ' and ', (m) => m.description);
}

class _Or<T> implements Matcher<T> {
  List<Matcher<T>> _delegates;

  _Or(List<Matcher<T>> this._delegates);

  bool matches(T value) {
    for (final m in _delegates) {
      if (m.matches(value)) {
        return true;
      }
    }
    return false;
  }

  String get description => join(_delegates, ' or ', (m) => m.description);
}

class _Not<T> implements Matcher<T> {
  Matcher<T> _delegate;

  _Not(Matcher<T> this._delegate);

  bool matches(T value) => !_delegate.matches(value);
  String get description => "not ${_delegate.description}";
}

/// Matches only if the value matches for all given [matchers].
Matcher and(List<Matcher> matchers) => new _And(matchers);

/// Matches only if the value matches for at least one of given [matchers].
Matcher or(List<Matcher> matchers) => new _Or(matchers);

/// Matches only if the value doesn't match for given [matcher].
Matcher not(Matcher matcher) => new _Not(matcher);

