// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _Caught {
  final bool something; // if something was caught
  final object;         // the object that was caught

  _Caught(this.something, this.object);
}

_Caught _throws(Function f) {
  try {
    f();
    return new _Caught(false, null);
  } catch (e) {
    return new _Caught(true, e);
  }
}

class _Throws implements Matcher<Function> {
  Matcher _exceptionMatcher;

  _Throws(this._exceptionMatcher);

  bool matches(Function f) {
    var caught = _throws(f);
    if (!caught.something) {
      return false;
    }

    return _exceptionMatcher == null || _exceptionMatcher.matches(caught.object);
  }

  String get description
      =>_exceptionMatcher == null ? "throw an exception"
      : "throw an exception which needs to ${_exceptionMatcher.description}";
}

class _ReturnsNormally implements Matcher<Function> {
  bool matches(Function f) => !_throws(f).something;
  String get description => "return normally";
}

/// Every [Function] that throws something (usually an [Exception],
/// but it can be any object) matches. Note that the function is invoked
/// exactly once.
///
/// Optionally, you can also specify a matcher for the [caught] object.
Matcher<Function> throws([Matcher caught]) => new _Throws(caught);

/// Every [Function] that returns normally (i.e., doesn't throw
/// anything) matches. Note that the function is invoked exactly once.
Matcher<Function> returnsNormally() => new _ReturnsNormally();

