// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

/// Predicate is a function that returns [bool] for given value.
typedef bool Predicate(value);

/// Creates a [Predicate] from given value. It can be [Predicate]
/// itself, in which case it is returned as is, or it can be
/// a [Matcher], in which case a [Predicate] is returned that
/// is [true] when the matcher matches. Throws exception otherwise.
Predicate positivePredicate(predicate) {
  if (predicate is Predicate) {
    return predicate;
  } else if (predicate is Matcher) {
    return (value) => predicate.matches(value);
  } else {
    throw new ExpectException('predicate must be Predicate or Matcher');
  }
}

/// Creates a negative [Predicate] from given value. It can be [Predicate]
/// itself, in which case a [Predicate] is returned that is [true]
/// when the predicate is [false], or it can be a [Matcher], in which case
/// a [Predicate] is returned that is [true] when the matcher _doesn't_ match.
/// Throws exception otherwise.
Predicate negativePredicate(predicate) {
  if (predicate is Predicate) {
    return (value) => !predicate(value);
  } else if (predicate is Matcher) {
    return (value) => !predicate.matches(value);
  } else {
    throw new ExpectException('predicate must be Predicate or Matcher');
  }
}

